--[[
## Title: Mini-Me
## Notes: Shows a 3D model of your character. Also displays bag space, equipment durability, etc. By CHiLLZ [v1.2]
## SavedVariablesPerCharacter: MM_Config
##
## Author: CHiLLZ <chillz@xmission.com> 
## License: Copyright 2006-2008, Chillz Media. All rights reserved.
]]--

-- SETTINGS --
local MM_Title = "Mini-Me"
local MM_Version = 1.2;
local MM_Debug = false;

local MM_Defaults = {
	["Version"] = MM_Version,
	["Options"] = {
		["MM_Player"] = "true",
		["MM_Block1"] = "true",
		["MM_Block2"] = "true",
	},
}

-- FUNCTIONS --
function MM_OnLoad()
	MM:RegisterForDrag("LeftButton");
	MM:RegisterEvent("ADDON_LOADED");
	
	MM:RegisterEvent("UNIT_MODEL_CHANGED");
	MM:RegisterEvent("UNIT_INVENTORY_CHANGED");
	MM:RegisterEvent("PLAYER_REGEN_DISABLED");
	MM:RegisterEvent("PLAYER_REGEN_ENABLED");
	MM:RegisterEvent("PLAYER_ENTER_COMBAT");
	MM:RegisterEvent("PLAYER_LEAVE_COMBAT");	
	
	MM:RegisterEvent("PLAYER_ENTERING_WORLD");
	MM:RegisterEvent("PLAYER_XP_UPDATE");
	MM:RegisterEvent("PLAYER_LEVEL_UP");
	
	MM:RegisterEvent("PLAYER_PVP_KILLS_CHANGED");
	
	MM:RegisterEvent("BAG_UPDATE");
end

function MM_OnEvent()
	if ((event == "ADDON_LOADED") and (arg1 == MM_Title)) then
		MM_Loader();
	end
	
	if (event == "UNIT_MODEL_CHANGED"
		or "UNIT_INVENTORY_CHANGED"
		or "PLAYER_REGEN_DISABLED"
		or "PLAYER_REGEN_ENABLED"
		or "PLAYER_ENTER_COMBAT"
		or "PLAYER_LEAVE_COMBAT"
	) then
		MM_Player:SetUnit("player");
	end

	if (event == "PLAYER_XP_UPDATE" or "PLAYER_LEVEL_UP") then
		MM_PlayerInfo();
	end
	
	if (event == "PLAYER_PVP_KILLS_CHANGED") then
		--MM_UpdatePVP();
	end
	
	if (event == "BAG_UPDATE") then
		MM_GetBags();
	end
	
	if (event == "PLAYER_ENTERING_WORLD"
		or "UNIT_INVENTORY_CHANGED"
		or "PLAYER_REGEN_ENABLED"
		or "PLAYER_LEAVE_COMBAT"
		or "PLAYER_MONEY"
	) then
		MM_Durability();
	end
end

function MM_OnShow()
	MM_Player:SetUnit("player");
	MM_Player:SetScale(1.2);
	MM_Player:SetPosition(0,0,0);
	
	MM_PlayerInfo();
	--MM_UpdatePVP();
	MM_GetBags();
	MM_Durability();
	
	if (MM_Debug) then
		MM_DebugInfo:SetText("Debugging display enabled.");
	end
end

function MM_DebugOut()
	if (MM_Debug and MM_Config["Options"]) then
		DEFAULT_CHAT_FRAME:AddMessage( "<Mini-Me>: \(debug\)" );
		DEFAULT_CHAT_FRAME:AddMessage( "     MM_Player = "..MM_Config["Options"]["MM_Player"] );
		DEFAULT_CHAT_FRAME:AddMessage( "     MM_Block1 = "..MM_Config["Options"]["MM_Block1"] );
		DEFAULT_CHAT_FRAME:AddMessage( "     MM_Block2 = "..MM_Config["Options"]["MM_Block2"] );
	end
end

function MM_Loader()
	-- reset the data file?
	if (not MM_Config) then
		MM_Config = MM_Defaults;
		DEFAULT_CHAT_FRAME:AddMessage( "Mini-Me v"..MM_Version.." initialized!  By CHiLLZ" );
	elseif (not MM_Config["Version"]) then
		MM_Config = MM_Defaults;
		DEFAULT_CHAT_FRAME:AddMessage( "Mini-Me v"..MM_Version.." upgraded!  By CHiLLZ" );
	elseif (MM_Version > MM_Config["Version"]) then
		MM_Config = MM_Defaults;
		DEFAULT_CHAT_FRAME:AddMessage( "Mini-Me v"..MM_Version.." upgraded.  By CHiLLZ" );
	else
		DEFAULT_CHAT_FRAME:AddMessage( "Mini-Me v"..MM_Version.." loaded.  By CHiLLZ" );
	end
	
	-- start the mod
	MM_Settings();
end

function MM_Settings()
	if (MM_Config["Options"]["MM_Player"] == "false") then MM_Player:Hide(); else MM_Player:Show(); end
	if (MM_Config["Options"]["MM_Block1"] == "false") then MM_Block1:Hide(); else MM_Block1:Show(); end
	if (MM_Config["Options"]["MM_Block2"] == "false") then MM_Block2:Hide(); else MM_Block2:Show(); end

	MM_DebugOut();
end

function MM_Toggle( element )
	if ( getglobal(element):IsVisible() ) then
		-- save setting
		MM_Config["Options"][element] = "false";
		-- apply the change
		getglobal(element):Hide();
		PlaySound("gsTitleOptionOK");
	else 
		-- save setting
		MM_Config["Options"][element] = "true";
		-- apply the change
		getglobal(element):Show();
		PlaySound("gsTitleOptionOK");
	end
	MM_DebugOut();
end

function MM_PlayerInfo()
	MM_Level:SetText( "["..UnitLevel("player").."]" );
	MM_Level:SetVertexColor(1, 1, 1);
	MM_Name:SetText( UnitName("player") );
	MM_Name:SetVertexColor(1, 1, 1);
	
	local faction = UnitFactionGroup("player");
	if (faction) then
		--MM_HonorIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-"..faction);
	end
end

function MM_GetBags()
	local used,total,free;
	used = 0;
	total = 0;
	free = 0;
	
	for i = 0,4 do
		local u,t = MM_GetSlots(i);
		used = used + u;
		total = total + t;
	end
	free = total - used;

	if (free < 2) then 
		MM_BagSpace:SetVertexColor(1, 0, 0);
		MM_BagIcon:SetTexture("Interface\\Icons\\INV_Misc_Bag_10_Red");
	elseif (free < 3) then
		MM_BagSpace:SetVertexColor(1, 0.5, 0);
		MM_BagIcon:SetTexture("Interface\\Icons\\INV_Misc_Bag_10");
	elseif (free < 5) then
		MM_BagSpace:SetVertexColor(1, 1, 0);
		MM_BagIcon:SetTexture("Interface\\Icons\\INV_Misc_Bag_10");
	elseif (free < 9) then
		MM_BagSpace:SetVertexColor(0, 1, 0);
		MM_BagIcon:SetTexture("Interface\\Icons\\INV_Misc_Bag_10");
	else 
		MM_BagSpace:SetVertexColor(1, 1, 1);
		MM_BagIcon:SetTexture("Interface\\Icons\\INV_Misc_Bag_10");
	end
	
	MM_BagSpace:SetText( free.."/"..total.." free");
end

function MM_GetSlots(bag)
	local numUsed, numSlots = 0, GetContainerNumSlots(bag);
	for i = 1,numSlots do
		if ( GetContainerItemLink(bag, i) ) then
			numUsed = numUsed + 1;
		end
	end
	return numUsed, numSlots;
end

function MM_Durability()
	local gearslots, cur, max, current, maximum, percent;
	local broken = false;
	current = 0;
	maximum = 0;
	percent = 0;
	
	gearslots = {"HeadSlot","ShoulderSlot","ChestSlot","WristSlot","HandsSlot","WaistSlot","LegsSlot","FeetSlot","MainHandSlot","SecondaryHandSlot","RangedSlot"};
	
	-- check each slot
	for i=1,11 do
		-- get info for each equipped item
		cur, max = MM_ItemDur(gearslots[i]); 
		
		-- only add if item has durability
		if (max) then
			-- is item broken?
			if (cur < 1) then
				broken = true;
			end
			
			-- append to totals
			current = current + cur; 
			maximum = maximum + max;
		end
	end
	
	-- calculate percentage
	if (maximum > 0) then
		percent = ( current / maximum ) * 100;
	else
		percent = 100;
	end
	
	-- colored text
	if (broken) then
		MM_Armor:SetVertexColor(1, 0, 0);
	elseif (percent < 21) then
		MM_Armor:SetVertexColor(1, 0, 0);
	elseif (percent < 41) then
		MM_Armor:SetVertexColor(1, 0.5, 0);
	elseif (percent < 61) then
		MM_Armor:SetVertexColor(1, 1, 0);
	elseif (percent < 81) then
		MM_Armor:SetVertexColor(0, 1, 0);
	else
		MM_Armor:SetVertexColor(1, 1, 1);
	end
	
	-- output
	percent = format("%.1f", percent);
	--MM_Armor:SetText(percent.."%  \("..current.."/"..maximum.."\)");
	MM_Armor:SetText(percent.."%");
end

function MM_ItemDur( index )
	MM_TT:SetOwner(WorldFrame, "ANCHOR_NONE");
	local id, hasItem, repairCost, current, maximum;
	
	id = GetInventorySlotInfo( index );
	hasItem, _, repairCost = MM_TT:SetInventoryItem("player", id);
	
	-- scan the item desc
	if (hasItem) then
		for i = 1,30 do
			local row = getglobal("MM_TTTextLeft" .. i);
			if (row) then
				local text = row:GetText();
				if (text) then
					-- search for durability string
					local _, _, cur, max = string.find(text, "Durability (%d+) / (%d+)");
					
					-- convert text to numeric
					if (cur) then
						current = tonumber(cur);
						maximum = tonumber(max);
					end
				end
			end
		end
	end
	
	return current, maximum;
end

function MM_UpdatePVP()
	local hk_lifetime, hk_today;
	
	hk_lifetime = GetPVPLifetimeStats();
	hk_today = GetPVPSessionStats();
	if (hk_today > 0) then
		MM_Kills:SetVertexColor(1, 1, 1);
	end
	
	MM_Kills:SetText(hk_lifetime.." \("..hk_today.." today\)");
end
